/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * ================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://forceten.tidalwave.it
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: GeoCoderEntityNode.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.node.impl;

import javax.annotation.Nonnull;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import it.tidalwave.util.IconProvider;
import it.tidalwave.netbeans.nodes.EnhancedNode;
import it.tidalwave.netbeans.nodes.SafeChildFactory;
import it.tidalwave.netbeans.nodes.role.DefaultNodeIconProvider;
import it.tidalwave.geo.geocoding.GeoCoderEntity;

/***********************************************************************************************************************
 *
 * A {@link Node} specialization for representing a {@link GeoEntity}.
 *
 * @author  Fabrizio Giudici
 * @version $Id: GeoCoderEntityNode.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
@edu.umd.cs.findbugs.annotations.SuppressWarnings("HE_INHERITS_EQUALS_USE_HASHCODE")
public final class GeoCoderEntityNode extends EnhancedNode
  {
    private final static IconProvider NODE_ICON_PROVIDER = 
            new DefaultNodeIconProvider("it/tidalwave/geo/geocoding/node/impl/icons/default.png");

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static class GeoCoderEntityChildFactory extends SafeChildFactory<GeoCoderEntity>
      {
        @Nonnull
        private final GeoCoderEntity geoCoderEntity;
        
        public GeoCoderEntityChildFactory (final @Nonnull GeoCoderEntity geoCoderEntity)
          {
            assert geoCoderEntity != null : "null geoCoderEntity";
            
            this.geoCoderEntity = geoCoderEntity;
          }

        @Override
        protected boolean createKeysSafely (final @Nonnull List<GeoCoderEntity> geoCoderEntities)
          {
            geoCoderEntities.addAll(geoCoderEntity.findChildren().results());
            return true;
          }
        
        @Override @Nonnull 
        protected Node createNodeForKey (final @Nonnull GeoCoderEntity geoCoderEntity)
          {
            return new GeoCoderEntityNode(geoCoderEntity);
          }
        
        public void refresh()
          {
            refresh(false);
          }
      };
            
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public GeoCoderEntityNode (final @Nonnull GeoCoderEntity geoCoderEntity)
      {
        this(geoCoderEntity, new GeoCoderEntityChildFactory(geoCoderEntity));
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private GeoCoderEntityNode (final @Nonnull GeoCoderEntity geoCoderEntity,
                                final @Nonnull GeoCoderEntityChildFactory childFactory)
      {
        super(geoCoderEntity, Children.create(childFactory, true), createLookup(geoCoderEntity));
        childFactory.setNode(this);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private static Lookup createLookup (final @Nonnull GeoCoderEntity geoCoderEntity)
      {
        assert geoCoderEntity.getLookup() != null : "Null Lookup for " + geoCoderEntity;
        return new ProxyLookup(geoCoderEntity.getLookup(), Lookups.fixed(geoCoderEntity, NODE_ICON_PROVIDER));
      }
  }
